//
//  SkinCache.m
//  CheckboxAndRadio
//
//  Created by James Walker on 1/3/14.
//  Copyright (c) 2014 James W. Walker.

// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from
// the use of this software.

// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:

// 1. The origin of this software must not be misrepresented; you must not
// claim that you wrote the original software. If you use this software in a
// product, an acknowledgment in the product documentation would be
// appreciated but is not required.

// 2. Altered source versions must be plainly marked as such, and must not be
// misrepresented as being the original software.

// 3. This notice may not be removed or altered from any source distribution.
//

#import "SkinCache.h"

@implementation SkinCache

static NSDictionary*			sSkinMap = nil;
static NSMutableDictionary*		sSkinCache = nil;


+ (void)initialize
{
	sSkinCache = [[NSMutableDictionary alloc] initWithCapacity: 100];
	
	NSURL* mapURL = [[NSBundle mainBundle] URLForResource: @"UISkin"
											withExtension: @"plist" ];
	
	if (mapURL == nil)
	{
		[NSException raise: @"SkinResMissing"
					format: @"Failed to find resource UISkin.plist."];
	}
	
	sSkinMap = [[NSDictionary alloc] initWithContentsOfURL: mapURL];
	
	if (sSkinMap == nil)
	{
		[NSException raise: @"SkinResBad"
					format: @"Failed to parse resource UISkin.plist."];
	}
}

+ (NSString*) nameOfState:  (NSInteger) buttonState
			highlight: (SkinHighlight) hilite
{
	NSString* name = @"";
	
	switch (buttonState)
	{
		case NSOnState:
			switch (hilite)
			{
				case kSkinHighlight_None:
					name = @"on";
					break;
					
				case kSkinHighlight_Pressed:
					name = @"on pressed";
					break;
					
				case kSkinHighlight_MouseOver:
					name = @"on mouseover";
					break;
			}
			break;
			
		case NSOffState:
			switch (hilite)
			{
				case kSkinHighlight_None:
					name = @"off";
					break;
					
				case kSkinHighlight_Pressed:
					name = @"off pressed";
					break;
					
				case kSkinHighlight_MouseOver:
					name = @"off mouseover";
					break;
			}
			break;
			
		case NSMixedState:
			switch (hilite)
			{
				case kSkinHighlight_None:
					name = @"mixed";
					break;
					
				case kSkinHighlight_Pressed:
					name = @"mixed pressed";
					break;
					
				case kSkinHighlight_MouseOver:
					name = @"mixed mouseover";
					break;
			}
			break;
	}
	
	return name;
}

/*!
	@function	imageForKey:state:highlight:
	
	@abstract	Get a cached image for a button in a particular state.
	
	@discussion	The name state, and highlight values you provide are looked up
				in UISkin.plist, giving a resource name which is in turn
				fetched using -[NSImage imageNamed:].
				
				The SkinCache retains the image, so you do not need to retain
				or release it.
				
				If no image is found for the given highlight state, the cache
				will try again using kSkinHighlight_None as a fallback value.

	@param		name		Name of a set of image resource names in
							UISkin.plist.
	@param		buttonState	State of the button being drawn.
	@param		hilite		Desired highlight state, from SkinHighlight.
	@result		An image, or nil if it is not found.
*/
+ (NSImage*) imageForKey: (NSString*) name
			state: (NSInteger) buttonState
			highlight: (SkinHighlight) hilite
{
	NSImage* foundImage = nil;
	NSDictionary* stateToIm = [sSkinCache objectForKey: name];
	
	if (stateToIm == nil)
	{
		NSDictionary* stateToName = [sSkinMap objectForKey: name];
		
		if ( (stateToName != nil) &&
			[stateToName isKindOfClass: [NSDictionary class]] )
		{
			NSMutableDictionary* newStateToIm = [NSMutableDictionary dictionary];
			
			for (NSString* oneKey in stateToName)
			{
				NSString* resName = [stateToName objectForKey: oneKey];
				
				if (! [resName isEqualToString: @""])
				{
					NSImage* anImage = [NSImage imageNamed: resName];
					if (anImage != nil)
					{
						[newStateToIm setObject: anImage forKey: oneKey];
					}
				}
			}
			
			[sSkinCache setObject: newStateToIm forKey: name];
			stateToIm = newStateToIm;
		}
	}
	
	if (stateToIm != nil)
	{
		NSString* nameOfState = [SkinCache nameOfState: buttonState highlight: hilite];
		foundImage = [stateToIm objectForKey: nameOfState];
		
		if ( (foundImage == nil) && (hilite != kSkinHighlight_None) )
		{
			nameOfState = [SkinCache nameOfState: buttonState
									highlight: kSkinHighlight_None];
			foundImage = [stateToIm objectForKey: nameOfState];
		}
	}
	
	return foundImage;
}


@end
